pub enum Prng {
Seeded {
choices: Vec<u8>,
uplc: PlutusData,
},
Replayed {
choices: Vec<u8>,
uplc: PlutusData,
},
}Expand description
—– PRNG —————————————————————–
A Pseudo-random generator (PRNG) used to produce random values for fuzzers. Note that the randomness isn’t actually managed by the Rust framework, it entirely relies on properties of hashing algorithm on-chain (e.g. blake2b).
The PRNG can have two forms:
-
Seeded: which occurs during the initial run of a property. Each time a number is drawn from the PRNG, a new seed is created. We retain all the choices drawn in a choices vector.
-
Replayed: which is used to replay a Prng sequenced from a list of known choices. This happens when shrinking an example. Instead of trying to shrink the value directly, we shrink the PRNG sequence with the hope that it will generate a smaller value. This implies that generators tend to generate smaller values when drawing smaller numbers.
Variants
Implementations
Sourceimpl Prng
impl Prng
pub fn uplc(&self) -> PlutusData
pub fn choices(&self) -> Vec<u8> ⓘ
Sourcepub fn from_choices(choices: &[u8]) -> Prng
pub fn from_choices(choices: &[u8]) -> Prng
Construct a Pseudo-random number generator from a pre-defined list of choices.
Sourcepub fn sample(
&self,
fuzzer: &Program<Name>,
) -> Result<Option<(Prng, PlutusData)>, FuzzerError>
pub fn sample( &self, fuzzer: &Program<Name>, ) -> Result<Option<(Prng, PlutusData)>, FuzzerError>
Generate a pseudo-random value from a fuzzer using the given PRNG.
Sourcepub fn from_result(result: Term<NamedDeBruijn>) -> Option<(Self, PlutusData)>
pub fn from_result(result: Term<NamedDeBruijn>) -> Option<(Self, PlutusData)>
Obtain a Prng back from a fuzzer execution. As a reminder, fuzzers have the following signature:
type Fuzzer<a> = fn(Prng) -> Option<(Prng, a)>
In nominal scenarios (i.e. when the fuzzer is made from a seed and evolve pseudo-randomly), it cannot yield ‘None’. When replayed however, we can’t easily guarantee that the changes made during shrinking aren’t breaking underlying invariants (if only, because we run out of values to replay). In such case, the replayed sequence is simply invalid and the fuzzer aborted altogether with ‘None’.